varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinateUpscale;
varying highp vec2 textureScale;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float blurOpacity;
uniform lowp float factor1;
uniform lowp float factor2;
uniform lowp float factor3;
uniform lowp float factor4;
uniform lowp float factor5;
uniform lowp float factor6;
uniform lowp float factor7;
uniform lowp float filterOpacity;

lowp vec3 rgb2hsv(lowp vec3 c) {
    lowp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    highp float d = q.x - min(q.w, q.y);
    highp float e = 1.0e-10;
    lowp vec3 hsv = vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
    return hsv;
}

lowp float hsvToInterpolation(lowp vec3 hsv) {
    lowp float op = 1.0;
    if (hsv.z < 0.3)
    {
        op = (hsv.z - 0.2) * 10.0;
    }
    
    if (hsv.x >= 0.91 || hsv.x <= 0.17)
    {
        return op;
    }
    else if (hsv.x >= 0.89)
    {
        return op * (hsv.x - 0.89) * 33.33;
    }
    else if (hsv.x <= 0.2)
    {
        return op * (0.2 - hsv.x) * 33.33;
    }
    
    return 0.0;
}

lowp vec3 ContrastSaturationBrightness(lowp vec3 color, lowp float brt, lowp float sat, lowp float con)
{
    const lowp float AvgLumR = 0.5;
    const lowp float AvgLumG = 0.5;
    const lowp float AvgLumB = 0.5;
    
    const lowp vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
    
    lowp vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
    lowp vec3 brtColor = color * brt;
    lowp vec3 intensity = vec3(dot(brtColor, LumCoeff));
    lowp vec3 satColor = mix(intensity, brtColor, sat);
    lowp vec3 conColor = mix(AvgLumin, satColor, con);
    
    return conColor;
}

void main() {
    lowp vec4 inputColor = texture2D(inputImageTexture, textureCoordinate);
    
    lowp vec3 hsv = rgb2hsv(inputColor.rgb);
    lowp float interpolationValue = hsvToInterpolation(hsv);
    
    lowp float brightLimitValue = 1.0;
    if (hsv.z <= 0.2)
    {
        brightLimitValue = 0.0;
    }
    interpolationValue = min(interpolationValue, brightLimitValue);
    
    lowp float hueLimitValue = 1.0;
    if (0.17 < hsv.x && hsv.x < 0.89)
    {
        hueLimitValue = 0.0;
    }
    interpolationValue = min(interpolationValue, hueLimitValue);
    
    if (interpolationValue == 0.0)
    {
        gl_FragColor = inputColor;
        return;
    }
    
    lowp vec4 blurColor = texture2D(inputImageTexture2, textureCoordinate);
    
    interpolationValue = blurOpacity * interpolationValue;
    
    lowp float cDistance = distance(vec3(0.0, 0.0, 0.0), max(blurColor.rgb - inputColor.rgb, 0.0)) * factor1;
    lowp vec3 brightColor = ContrastSaturationBrightness(inputColor.rgb, factor2, 1.0, factor3);
    lowp vec3 mix11Color = mix(inputColor.rgb, brightColor.rgb, cDistance);
    
    lowp float dDistance = distance(vec3(0.0, 0.0, 0.0), max(inputColor.rgb-blurColor.rgb, 0.0)) * factor4;
    lowp vec3 darkColor = ContrastSaturationBrightness(inputColor.rgb, factor5, 1.0, factor6);
    lowp vec3 mix115Color = mix(mix11Color.rgb, darkColor.rgb, dDistance);
    
    lowp vec3 mix12Color;
    
    if (factor7 < 0.999)
    {
        lowp vec3 mix116Color = mix(inputColor.rgb, mix115Color.rgb, factor7);
        mix12Color = mix(mix116Color.rgb, blurColor.rgb, interpolationValue);
    }
    else
    {
        mix12Color = mix(mix115Color.rgb, blurColor.rgb, interpolationValue);
    }
    
    if (filterOpacity < 0.999)
    {
        gl_FragColor = vec4(mix(inputColor.rgb, mix12Color.rgb, filterOpacity), 1.0);
    }
    else
    {
        gl_FragColor = vec4(mix12Color.rgb, 1.0);
    }
}