#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

///////////////////////////////////////////////////////////
// Uniforms
uniform sampler2D u_texture;
uniform float aspectRatio;
uniform vec2 center;
uniform vec2 radius;
uniform float scale;
uniform float angle;

uniform float u_min;
uniform float u_max;
uniform int debug;


uniform float progress;
///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;



void main()
{

    if(scale == 0.0) {
        gl_FragColor = texture2D(u_texture, v_texCoord );
        return;;
    }
    
    vec2 textureCoordinateToUse = v_texCoord;

    float e1 =  ( textureCoordinateToUse.x - center.x ) / ( radius.x );
    float e2 =  ( textureCoordinateToUse.y - center.y ) / ( radius.y / aspectRatio );
    float d  = (e1 * e1) + (e2 * e2);
    
    
    
    gl_FragColor = texture2D(u_texture, textureCoordinateToUse );
    if (d < 1.0)
    {
        
        float dist = 1.0 - d;
        float delta = scale * dist * progress;
        
        float deltaScale = smoothstep(u_min, u_max, dist);
        float directionX = cos(angle) * deltaScale;
        float directionY = sin(angle) * deltaScale;
        
        gl_FragColor = texture2D(u_texture,
                                 vec2(textureCoordinateToUse.x - (delta * directionX),
                                      textureCoordinateToUse.y -  (delta * directionY))
                                 );
        
        if(debug == 1) {
            gl_FragColor = mix(gl_FragColor, vec4(1.0, 0, 0, 1.0), deltaScale);
            if(d < 0.0025) gl_FragColor = mix(gl_FragColor, vec4(0.0, 1.0, 1.0, 1.0), 1.0);
        }

        
        
    }
    
    if(debug == 1 && d < 1.0 && d > 0.95) {
        gl_FragColor = mix(gl_FragColor, vec4(1.0, 0, 0, 1.0), 1.0);
        
    }
    


}
