#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

#define MAXPOINTS 75

uniform vec2 p1[MAXPOINTS]; // where the drag started
uniform vec2 p2[MAXPOINTS]; // where the drag ended
uniform float sizes[MAXPOINTS];
uniform lowp int pointCount;
uniform float progress;


///////////////////////////////////////////////////////////
// Uniforms
uniform sampler2D u_texture;

///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;
varying vec2 v_texCoord_origin;

void main()
{
    vec2 uv = v_texCoord_origin * 2.0 - 1.0;
    gl_FragColor = texture2D(u_texture, v_texCoord);

    
    vec2 delta = vec2(0.0, 0.0);
    for (int i = 0; i < pointCount; i++)
    {
        vec2 point1 = p1[i];
        vec2 point2 = p2[i];
        

        float dragdistance = distance(p1[i], p2[i]);
        
        vec2 current = uv;
  
        float e1 =  ( current.x - point1.x ) / ( dragdistance );
        float e2 =  ( current.y - point1.y ) / ( dragdistance / 1.333333333 );
        float d  = sqrt( (e1 * e1) + (e2 * e2));
        
        float mydistance = distance(p1[i], current);
        
        if (d < 1.0 && d > 0.95)
        {
          
            gl_FragColor = mix(gl_FragColor, vec4(1.0, 0, 0, 1.0), 1.0);
            return;
        }
    }

    
}